//
// Authors:  Dean Hinshaw - University of Leicester
//
// DSS test : Construct a DSS specification and test that it produces
//            a proper selectlib expression
//


#include <cstring>

#include <Dal.h>
#include <Region.h>
#include <DataSubspace.h>
#include <errstr.h>
#include <DataFilter.h>
/* AI gcc4 porting
 * 'abs' was not declare in this scope
 */
#include <cmath>
int main(){

  dataSetServer->copy("emlist.fits", "templist.fits");
  DataSet * elist = dataSetServer->open("templist.fits", DataSet::Modify);
  Table * etab = elist->table("EVENTS");

  DataSubspace dss;
  DScomp * comp1 = dss.addComponent();
  DScomp * comp2 = dss.addComponent();

  //AI: 05/12/2003
  GeoRegion regions;
  regions.push_back(Component());
  regions.push_back(Component());
  //AI: 05/12/2003

  // Range filter

  list<Range<long> > ranges1;
  Range<long> range1((long)0,(long)200);
  ranges1.push_back(range1);
  Range<long> range2((long)300,(long)600,Open,Open);
  ranges1.push_back(range2);

  comp1->addRangeFilter("RAWX", ranges1, "PIXELS");
  comp2->addRangeFilter(etab->column("RAWY"), ranges1);

  // GTI filter

  comp1->addBlockFilter("TIME", "gti.fits:STDGTI", "SECONDS");

  // ASC region filter

  DSS::DSfilterComp comps;
  comps.push_back("X");
  comps.push_back("Y");

  comp1->addBlockFilter("SOURCE", "region.fits:REGION", comps);


  Point<float> refpoint(25000,38000);
  Point<float> refpoint1(31000,12500);

  //AI: 05/12/2003
  //Commented due to runtime error with gcc 3.3
  //The error is related with selectlib package
  //particulary with the Region.h file
  // Region filter
  //
  //  GeoRegion regions;
  //
  //  regions.push_back(Component());
  //  Point<float> refpoint(25000,38000);
  //
  //  regions[0].push_back( new FCircle(refpoint, 10000, true) );
  //  regions[0].push_back( new FSector(refpoint, 45, 135, true) );
  //
  //  regions.push_back(Component());
  //  Point<float> refpoint1(31000,12500);


  regions[0].push_back( new FCircle(refpoint, 10000, true) );
  regions[0].push_back( new FSector(refpoint, 45, 135, true) );

  regions[1].push_back( new FElliptAnnulus(refpoint1, 100, 150, 200, 250, 90, 45, true) );
  regions[1].push_back( new FBox(refpoint1, 2500, 5000, false) );

  regions.push_back(Component());
  vector<float> X, Y;
  X.push_back(1000);
  X.push_back(1100);
  X.push_back(1000);
  X.push_back(900);
  X.push_back(1000);

  Y.push_back(1000);
  Y.push_back(900);
  Y.push_back(800);
  Y.push_back(900);
  Y.push_back(1000);

  regions[2].push_back( new FPolygon(X, Y, 5, true) );

  comp2->addRegionFilter("POS", regions);

  // See if we can get back the range values

  string axis = "RAWX";
  list<Range<double> > ranges2 = dss.ranges(axis);

  if( *ranges1.begin() != *(ranges2.begin()) || *(++ranges1.begin()) != *(++ranges2.begin()) )
    errstr << "dsslib failed range retrieval." << Sas::error("Ranges");

  // Now see if we can make a valid selectlib expression out of this mess

  DataFilter filter1(dss.selectExpression());
  if( filter1.doSelect(*etab) != 1104 )
    errstr << "dsslib failed test expression 1." << Sas::error("Express1");

  DataSubspace dssNeg = dss;

  !dssNeg;
  DataFilter filter1_5(dssNeg.selectExpression());
  if( filter1_5.doSelect(*etab) != 1075 )
    errstr << "dsslib failed test expression 1.5" << Sas::error("Express1.5");

  // Bitmask filter

  comp2->addBitFilter(etab->column("FLAG"), "b000000000000");

  // Image mask filter

  comp2->addBlockFilter("DET", "maskwcs.fits:MASK");

  DataFilter filter1_6(dss.selectExpression());
  if( filter1_6.doSelect(*etab) != 1083 )
    errstr << "dsslib failed test expression 1.6" << Sas::error("Express1.6");

  DSS::DSaxes axes;
  axes.push_back("TIME");

  DataSubspace dsstemp2 = dss;
  dsstemp2.deleteComponent(1);

  DataFilter filter2(dsstemp2.selectExpression(axes));
  if( filter2.doSelect(*etab) != 2037 )
    errstr << "dsslib failed test expression 2." << Sas::error("Express2");

  double x, y;
  RangeBound Xb, Yb;
  dsstemp2.extent("X").values(x, y, Xb, Yb);

  Range<double> r(-15820,101820);
  if( r != dsstemp2.extent("X") )
    errstr << "dsslib failed extent test." << Sas::error("Extent");

  //if( r != dsstemp2.realextent("X") )
  dsstemp2.realextent("X").values(x, y, Xb, Yb);
  if ( abs(x+15820.0)>1.0 || abs(y-101820.0)>1.0 )
    errstr << "dsslib failed realextent test." << Sas::error("Extent");

  axes.clear();
  axes.push_back("POS");
  axes.push_back("SOURCE");

  DataFilter filter3(dss.selectExpression(axes));
  if( filter3.doSelect(*etab) != 1215 )
    errstr << "dsslib failed test expression 3." << Sas::error("Express3");

  // Write out the DSS to a fits table

  list<Range<long> > ranges3;
  Range<long> range3((long)0,(long)10000);
  ranges3.push_back(range3);

  comp2->addRangeFilter("RAWX", ranges3, "PIXELS");

  comp1->addBitFilter(etab->column("FLAG"), "b000000000000");

  comp1->addRegionFilter("POS", regions);

  dss.addComponent()->addBlockFilter("DET", "maskwcs.fits:MASK");

  list<Range<double> > ranges4;
  Range<double> range4(0.,10000.);
  ranges4.push_back(range4);
  comp1->addRangeFilter("TIME", ranges4, "SECONDS");

  etab << dss;
  dataSetServer->close(elist);

  elist = dataSetServer->open("templist.fits", DataSet::Modify);
  etab = elist->table("EVENTS");

  // Now try reading it back in.

  DataSubspace dss2(etab);

  // Make sure we can delete stuff

  dss2.component(1)->deleteFilter(0);
  dss2.deleteComponent(2);
  dss2.deleteComponent(0);

  DataFilter filter4(dss2.selectExpression());
  if( filter4.doSelect(*etab) != 33 )
    errstr << "dsslib failed test expression 4." << Sas::error("Express4");

  dss2.component(0)->deleteFilter(1);

  DataFilter filter5(dss2.selectExpression());
  if( filter5.doSelect(*etab) != 1516 )
    errstr << "dsslib failed test expression 5." << Sas::error("Express5");

  dataSetServer->close(elist);
  return 0;
}

